// Модудь, эмулирующий сражение двух бойцов
package fighter

import (
	"encoding/json"
	"fmt"
	"io/ioutil"
	"log"
	"math/rand"
	"time"
)

type Punch struct {
	Name   string
	Damage int
}

// Загружает список приемов и их урон из json файла со структурой
// `[{"name": "p_name_1", "damage": 5},{"name": "p_name_2", "damage": 8}]`
func loadPunches(filname string) []Punch {
	cont, err := ioutil.ReadFile(filname)
	if err != nil {
		log.Fatal("Error while reading file: ", err)
	}

	var punches []Punch

	err = json.Unmarshal(cont, &punches)

	if err != nil {
		log.Fatal("Error during Unmarshal(): ", err)
	}

	return punches
}

// Устраивает драку.
// Бойцы бьют друг друга по очереди, пока у них есть здоровье.
func StartFight(firstFighter, secondFighter *Fighter) *Fighter {
	var winner *Fighter
	var choice int

	StrengthPunches := loadPunches("./cmd/fight_club/fighter/data/str_punches.json")

	for firstFighter.health > 0 && secondFighter.health > 0 {
		time.Sleep(time.Second)
		choice = choosePunch(firstFighter.strength, StrengthPunches)
		firstFighter.getPunch(&StrengthPunches[choice])

		if firstFighter.health == 0 {
			break
		}

		time.Sleep(time.Second)
		choice = choosePunch(secondFighter.strength, StrengthPunches)
		secondFighter.getPunch(&StrengthPunches[choice])

		fmt.Println()
	}

	if firstFighter.health > 0 {
		winner = firstFighter
	} else {
		winner = secondFighter
	}

	return winner
}

// Выбор приема на основе характеристики.
// Генерируется случайным образом от 0 до punchCharacteristic
func choosePunch(punchCharacteristic byte, punchList []Punch) int {
	var choice int

	if int(punchCharacteristic) <= len(punchList) {
		choice = rand.Intn(int(punchCharacteristic) - 1)
	} else {
		choice = rand.Intn(len(punchList))
	}

	return choice

}

// Боец получает удар, теряет здоровье
func (f *Fighter) getPunch(punch *Punch) {
	if punch.Damage > int(f.health) {
		f.health = 0
		fmt.Printf("%v получает урон от %v. HP: %d\n",
			f.name, punch.Name, f.health)
	} else {
		f.health -= byte(punch.Damage)
		fmt.Printf("%v получает урон от %v. HP: %d\n",
			f.name, punch.Name, f.health)
	}

}
